@@ -33,6 +33,7 @@
 #include <string.h>
 #include <setjmp.h>
 #include <jpeglib.h>
+#include <jerror.h>
 #include "gdk-pixbuf-private.h"
 #include "gdk-pixbuf-io.h"
 
@@ -110,7 +111,9 @@ fatal_error_handler (j_common_ptr cinfo)
         if (errmgr->error && *errmgr->error == NULL) {
                 g_set_error (errmgr->error,
                              GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                             cinfo->err->msg_code == JERR_OUT_OF_MEMORY 
+			     ? GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY 
+			     : GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
                              _("Error interpreting JPEG image file (%s)"),
                              buffer);
         }
@@ -543,6 +546,66 @@ gdk_pixbuf__jpeg_image_stop_load (gpointer data, GError **error)
 }
 
 
+static gboolean
+gdk_pixbuf__jpeg_image_load_lines (JpegProgContext  *context,
+                                   GError          **error)
+{
+        struct jpeg_decompress_struct *cinfo = &context->cinfo;
+        guchar *lines[4];
+        guchar **lptr;
+        guchar *rowptr;
+        gint   nlines, i;
+
+        /* keep going until we've done all scanlines */
+        while (cinfo->output_scanline < cinfo->output_height) {
+                lptr = lines;
+                rowptr = context->dptr;
+                for (i=0; i < cinfo->rec_outbuf_height; i++) {
+                        *lptr++ = rowptr;
+                        rowptr += context->pixbuf->rowstride;
+                }
+
+                nlines = jpeg_read_scanlines (cinfo, lines,
+                                              cinfo->rec_outbuf_height);
+                if (nlines == 0)
+                        break;
+
+                switch (cinfo->out_color_space) {
+                case JCS_GRAYSCALE:
+                        explode_gray_into_buf (cinfo, lines);
+                        break;
+                case JCS_RGB:
+                        /* do nothing */
+                        break;
+                case JCS_CMYK:
+                        convert_cmyk_to_rgb (cinfo, lines);
+                        break;
+                default:
+                        if (error && *error == NULL) {
+                                g_set_error (error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_UNKNOWN_TYPE,
+                                             _("Unsupported JPEG color space (%s)"),
+                                             colorspace_name (cinfo->out_color_space));
+                        }
+
+                        return FALSE;
+                }
+
+                context->dptr += nlines * context->pixbuf->rowstride;
+
+                /* send updated signal */
+		if (context->updated_func)
+			(* context->updated_func) (context->pixbuf,
+						   0,
+						   cinfo->output_scanline - 1,
+						   cinfo->image_width,
+						   nlines,
+						   context->user_data);
+        }
+
+        return TRUE;
+}
 
 
 /*
@@ -683,15 +746,16 @@ gdk_pixbuf__jpeg_image_load_increment (gpointer data,
 			context->dptr = context->pixbuf->pixels;
 			
 			/* Notify the client that we are ready to go */
-			(* context->prepared_func) (context->pixbuf,
-                                                    NULL,
-						    context->user_data);
+			if (context->prepared_func)
+				(* context->prepared_func) (context->pixbuf,
+							    NULL,
+							    context->user_data);
 			
 		} else if (!context->did_prescan) {
 			int rc;			
 			
 			/* start decompression */
-			cinfo->buffered_image = TRUE;
+			cinfo->buffered_image = cinfo->progressive_mode;
 			rc = jpeg_start_decompress (cinfo);
 			cinfo->do_fancy_upsampling = FALSE;
 			cinfo->do_block_smoothing = FALSE;
@@ -700,12 +764,20 @@ gdk_pixbuf__jpeg_image_load_increment (gpointer data,
 				continue;
 
 			context->did_prescan = TRUE;
+		} else if (!cinfo->buffered_image) {
+                        /* we're decompressing unbuffered so
+                         * simply get scanline by scanline from jpeg lib
+                         */
+                        if (! gdk_pixbuf__jpeg_image_load_lines (context,
+                                                                 error))
+                                return FALSE;
+
+			if (cinfo->output_scanline >= cinfo->output_height)
+				return TRUE;
 		} else {
-			/* we're decompressing so feed jpeg lib scanlines */
-			guchar *lines[4];
-			guchar **lptr;
-			guchar *rowptr;
-			gint   nlines, i;
+                        /* we're decompressing buffered (progressive)
+                         * so feed jpeg lib scanlines
+                         */
 
 			/* keep going until we've done all passes */
 			while (!jpeg_input_complete (cinfo)) {
@@ -717,53 +789,13 @@ gdk_pixbuf__jpeg_image_load_increment (gpointer data,
 					else
 						break;
 				}
-				/* keep going until we've done all scanlines */
-				while (cinfo->output_scanline < cinfo->output_height) {
-					lptr = lines;
-					rowptr = context->dptr;
-					for (i=0; i < cinfo->rec_outbuf_height; i++) {
-						*lptr++ = rowptr;
-						rowptr += context->pixbuf->rowstride;
-					}
-					
-					nlines = jpeg_read_scanlines (cinfo, lines,
-								      cinfo->rec_outbuf_height);
-					if (nlines == 0)
-						break;
 
-					switch (cinfo->out_color_space) {
-					    case JCS_GRAYSCALE:
-						    explode_gray_into_buf (cinfo, lines);
-						    break;
-					    case JCS_RGB:
-						    /* do nothing */
-						    break;
-					    case JCS_CMYK:
-						    convert_cmyk_to_rgb (cinfo, lines);
-						    break;
-					    default:
-						    if (error && *error == NULL) {
-							    g_set_error (error,
-									 GDK_PIXBUF_ERROR,
-									 GDK_PIXBUF_ERROR_UNKNOWN_TYPE,
-									 _("Unsupported JPEG color space (%s)"),
-									 colorspace_name (cinfo->out_color_space)); 
-						    }
-						    
-						    return FALSE;
-					}
+                                /* get scanlines from jpeg lib */
+                                if (! gdk_pixbuf__jpeg_image_load_lines (context,
+                                                                         error))
+                                        return FALSE;
 
-					context->dptr += nlines * context->pixbuf->rowstride;
-					
-				        /* send updated signal */
-					(* context->updated_func) (context->pixbuf,
-								   0, 
-								   cinfo->output_scanline-1,
-								   cinfo->image_width, 
-								   nlines,
-								   context->user_data);
-				}
-				if (cinfo->output_scanline >= cinfo->output_height && 
+				if (cinfo->output_scanline >= cinfo->output_height &&
 				    jpeg_finish_output (cinfo))
 					context->in_output = FALSE;
 				else
@@ -872,6 +904,7 @@ real_save_jpeg (GdkPixbuf          *pixbuf,
        int i, j;
        int w, h = 0;
        int rowstride = 0;
+       int n_channels;
        struct error_handler_data jerr;
        ToFunctionDestinationManager to_callback_destmgr;
 
@@ -922,6 +955,7 @@ real_save_jpeg (GdkPixbuf          *pixbuf,
        }
        
        rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+       n_channels = gdk_pixbuf_get_n_channels (pixbuf);
 
        w = gdk_pixbuf_get_width (pixbuf);
        h = gdk_pixbuf_get_height (pixbuf);
@@ -994,7 +1028,7 @@ real_save_jpeg (GdkPixbuf          *pixbuf,
        while (cinfo.next_scanline < cinfo.image_height) {
                /* convert scanline from ARGB to RGB packed */
                for (j = 0; j < w; j++)
-                       memcpy (&(buf[j*3]), &(ptr[i*rowstride + j*3]), 3);
+                       memcpy (&(buf[j*3]), &(ptr[i*rowstride + j*n_channels]), 3);
 
                /* write scanline */
                jbuf = (JSAMPROW *)(&buf);
@@ -1069,6 +1103,6 @@ MODULE_ENTRY (jpeg, fill_info) (GdkPixbufFormat *info)
 	info->description = N_("The JPEG image format");
 	info->mime_types = mime_types;
 	info->extensions = extensions;
-	info->flags = GDK_PIXBUF_FORMAT_WRITABLE;
+	info->flags = GDK_PIXBUF_FORMAT_WRITABLE | GDK_PIXBUF_FORMAT_THREADSAFE;
 	info->license = "LGPL";
 }
