@@ -84,7 +84,7 @@ static const GdkPointerHooks singlehead_default_pointer_hooks = {
 
 static const GdkPointerHooks *singlehead_current_pointer_hooks = &singlehead_default_pointer_hooks;
 
-G_DEFINE_TYPE (GdkDisplay, gdk_display, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GdkDisplay, gdk_display, G_TYPE_OBJECT)
 
 static void
 gdk_display_class_init (GdkDisplayClass *class)
@@ -305,7 +305,7 @@ gdk_keyboard_ungrab (guint32 time)
 /**
  * gdk_beep:
  * 
- * Emits a short beep.
+ * Emits a short beep on the default display.
  **/
 void
 gdk_beep (void)
