@@ -183,6 +183,8 @@ static GdkRegion*   gdk_window_get_visible_region     (GdkDrawable *drawable);
 
 static void gdk_window_free_paint_stack (GdkWindow *window);
 
+static void gdk_window_init       (GdkWindowObject      *window);
+static void gdk_window_class_init (GdkWindowObjectClass *klass);
 static void gdk_window_finalize   (GObject              *object);
 static void gdk_window_clear_backing_rect (GdkWindow *window,
 					   gint       x,
@@ -190,7 +192,24 @@ static void gdk_window_clear_backing_rect (GdkWindow *window,
 					   gint       width,
 					   gint       height);
 
-G_DEFINE_TYPE (GdkWindowObject, gdk_window_object, GDK_TYPE_DRAWABLE)
+static gpointer parent_class = NULL;
+
+GType
+gdk_window_object_get_type (void)
+{
+  static GType object_type = 0;
+
+  if (!object_type)
+    object_type = g_type_register_static_simple (GDK_TYPE_DRAWABLE,
+						 "GdkWindow",
+						 sizeof (GdkWindowObjectClass),
+						 (GClassInitFunc) gdk_window_class_init,
+						 sizeof (GdkWindowObject),
+						 (GInstanceInitFunc) gdk_window_init,
+						 0);
+  
+  return object_type;
+}
 
 GType
 _gdk_paintable_get_type (void)
@@ -199,7 +218,7 @@ _gdk_paintable_get_type (void)
 
   if (!paintable_type)
     {
-      static const GTypeInfo paintable_info =
+      const GTypeInfo paintable_info =
       {
 	sizeof (GdkPaintableIface),  /* class_size */
 	NULL,                        /* base_init */
@@ -217,7 +236,7 @@ _gdk_paintable_get_type (void)
 }
 
 static void
-gdk_window_object_init (GdkWindowObject *window)
+gdk_window_init (GdkWindowObject *window)
 {
   /* 0-initialization is good for all other fields. */
 
@@ -229,11 +248,13 @@ gdk_window_object_init (GdkWindowObject *window)
 }
 
 static void
-gdk_window_object_class_init (GdkWindowObjectClass *klass)
+gdk_window_class_init (GdkWindowObjectClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GdkDrawableClass *drawable_class = GDK_DRAWABLE_CLASS (klass);
   
+  parent_class = g_type_class_peek_parent (klass);
+
   object_class->finalize = gdk_window_finalize;
 
   drawable_class->create_gc = gdk_window_create_gc;
@@ -287,7 +308,7 @@ gdk_window_finalize (GObject *object)
   g_object_unref (obj->impl);
   obj->impl = NULL;
   
-  G_OBJECT_CLASS (gdk_window_object_parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -1717,6 +1738,15 @@ gdk_window_draw_glyphs_transformed (GdkDrawable      *drawable,
 	  tmp_matrix.y0 -= y_offset;
 	  matrix = &tmp_matrix;
 	}
+      else if (GDK_PANGO_UNITS_OVERFLOWS (x_offset, y_offset))
+	{
+	  PangoMatrix identity = PANGO_MATRIX_INIT;
+	  
+	  tmp_matrix = identity;
+	  tmp_matrix.x0 -= x_offset;
+	  tmp_matrix.y0 -= y_offset;
+	  matrix = &tmp_matrix;
+	}
       else
 	{
 	  x -= x_offset * PANGO_SCALE;
@@ -2914,7 +2944,8 @@ gdk_window_constrain_size (GdkGeometry *geometry,
  * @mask: return location for modifier mask
  *
  * Obtains the current pointer position and modifier state.
- * The position is given in coordinates relative to @window.
+ * The position is given in coordinates relative to the upper left 
+ * corner of @window.
  * 
  * Return value: the window containing the pointer (as with
  * gdk_window_at_pointer()), or %NULL if the window containing the
