@@ -107,8 +107,6 @@ struct _NodeUIReference
   GQuark action_quark;
 };
 
-static void        gtk_ui_manager_class_init      (GtkUIManagerClass *class);
-static void        gtk_ui_manager_init            (GtkUIManager      *self);
 static void        gtk_ui_manager_finalize        (GObject           *object);
 static void        gtk_ui_manager_set_property    (GObject           *object,
                                                    guint              prop_id,
@@ -163,52 +161,17 @@ enum
   PROP_UI
 };
 
-static GObjectClass *parent_class = NULL;
 static guint ui_manager_signals[LAST_SIGNAL] = { 0 };
 
-static GMemChunk *merge_node_chunk = NULL;
-
-GType
-gtk_ui_manager_get_type (void)
-{
-  static GtkType type = 0;
-
-  if (!type)
-    {
-      static const GTypeInfo type_info =
-      {
-        sizeof (GtkUIManagerClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) gtk_ui_manager_class_init,
-        (GClassFinalizeFunc) NULL,
-        NULL,
-        
-        sizeof (GtkUIManager),
-        0, /* n_preallocs */
-        (GInstanceInitFunc) gtk_ui_manager_init,
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT,
-				     "GtkUIManager",
-				     &type_info, 0);
-    }
-  return type;
-}
+G_DEFINE_TYPE (GtkUIManager, gtk_ui_manager, G_TYPE_OBJECT)
 
 static void
 gtk_ui_manager_class_init (GtkUIManagerClass *klass)
 {
   GObjectClass *gobject_class;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
 
-  if (!merge_node_chunk)
-    merge_node_chunk = g_mem_chunk_create (Node, 64,
-					   G_ALLOC_AND_FREE);
-
   gobject_class->finalize = gtk_ui_manager_finalize;
   gobject_class->set_property = gtk_ui_manager_set_property;
   gobject_class->get_property = gtk_ui_manager_get_property;
@@ -239,7 +202,7 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
 				   g_param_spec_string ("ui",
  							P_("Merged UI definition"),
 							P_("An XML string describing the merged UI"),
-							"<ui>\n</ui>",
+							"<ui>\n</ui>\n",
 							GTK_PARAM_READABLE));
 
 
@@ -255,7 +218,7 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
    * Since: 2.4
    */
   ui_manager_signals[ADD_WIDGET] =
-    g_signal_new ("add_widget",
+    g_signal_new (I_("add_widget"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
 		  G_STRUCT_OFFSET (GtkUIManagerClass, add_widget),
@@ -274,7 +237,7 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
    * Since: 2.4
    */
   ui_manager_signals[ACTIONS_CHANGED] =
-    g_signal_new ("actions_changed",
+    g_signal_new (I_("actions_changed"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
 		  G_STRUCT_OFFSET (GtkUIManagerClass, actions_changed),  
@@ -298,7 +261,7 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
    * Since: 2.4
    */
   ui_manager_signals[CONNECT_PROXY] =
-    g_signal_new ("connect_proxy",
+    g_signal_new (I_("connect_proxy"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
 		  G_STRUCT_OFFSET (GtkUIManagerClass, connect_proxy),
@@ -320,7 +283,7 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
    * Since: 2.4
    */
   ui_manager_signals[DISCONNECT_PROXY] =
-    g_signal_new ("disconnect_proxy",
+    g_signal_new (I_("disconnect_proxy"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
 		  G_STRUCT_OFFSET (GtkUIManagerClass, disconnect_proxy),
@@ -344,7 +307,7 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
    * Since: 2.4
    */
   ui_manager_signals[PRE_ACTIVATE] =
-    g_signal_new ("pre_activate",
+    g_signal_new (I_("pre_activate"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
 		  G_STRUCT_OFFSET (GtkUIManagerClass, pre_activate),
@@ -367,7 +330,7 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
    * Since: 2.4
    */
   ui_manager_signals[POST_ACTIVATE] =
-    g_signal_new ("post_activate",
+    g_signal_new (I_("post_activate"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
 		  G_STRUCT_OFFSET (GtkUIManagerClass, post_activate),
@@ -434,7 +397,7 @@ gtk_ui_manager_finalize (GObject *object)
   g_object_unref (self->private_data->accel_group);
   self->private_data->accel_group = NULL;
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gtk_ui_manager_parent_class)->finalize (object);
 }
 
 static void
@@ -470,7 +433,7 @@ gtk_ui_manager_get_property (GObject         *object,
       g_value_set_boolean (value, self->private_data->add_tearoffs);
       break;
     case PROP_UI:
-      g_value_set_string (value, gtk_ui_manager_get_ui (self));
+      g_value_take_string (value, gtk_ui_manager_get_ui (self));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -761,30 +724,36 @@ gtk_ui_manager_get_widget (GtkUIManager *self,
   return GTK_UI_MANAGER_GET_CLASS (self)->get_widget (self, path);
 }
 
+typedef struct {
+  GtkUIManagerItemType types;
+  GSList *list;
+} ToplevelData;
+
 static void
 collect_toplevels (GNode   *node, 
 		   gpointer user_data)
 {
-  struct {
-    GtkUIManagerItemType types;
-    GSList *list;
-  } *data = user_data;
+  ToplevelData *data = user_data;
 
-  switch (NODE_INFO (node)->type) {
-  case NODE_TYPE_MENUBAR:
-    if (data->types & GTK_UI_MANAGER_MENUBAR)
-      data->list = g_slist_prepend (data->list, NODE_INFO (node)->proxy);
-    break;
-  case NODE_TYPE_TOOLBAR:
-    if (data->types & GTK_UI_MANAGER_TOOLBAR)
-      data->list = g_slist_prepend (data->list, NODE_INFO (node)->proxy);
-    break;
-  case NODE_TYPE_POPUP:
-    if (data->types & GTK_UI_MANAGER_POPUP)
-      data->list = g_slist_prepend (data->list, NODE_INFO (node)->proxy);
-    break;
-  default: ;
-  }
+  if (NODE_INFO (node)->proxy)
+    {
+      switch (NODE_INFO (node)->type) 
+	{
+	case NODE_TYPE_MENUBAR:
+	  if (data->types & GTK_UI_MANAGER_MENUBAR)
+	data->list = g_slist_prepend (data->list, NODE_INFO (node)->proxy);
+	  break;
+	case NODE_TYPE_TOOLBAR:
+      if (data->types & GTK_UI_MANAGER_TOOLBAR)
+	data->list = g_slist_prepend (data->list, NODE_INFO (node)->proxy);
+      break;
+	case NODE_TYPE_POPUP:
+	  if (data->types & GTK_UI_MANAGER_POPUP)
+	    data->list = g_slist_prepend (data->list, NODE_INFO (node)->proxy);
+	  break;
+	default: ;
+	}
+    }
 }
 
 /**
@@ -805,10 +774,7 @@ GSList *
 gtk_ui_manager_get_toplevels (GtkUIManager         *self,
 			      GtkUIManagerItemType  types)
 {
-  struct {
-    GtkUIManagerItemType types;
-    GSList *list;
-  } data;
+  ToplevelData data;
 
   g_return_val_if_fail (GTK_IS_UI_MANAGER (self), NULL);
   g_return_val_if_fail ((~(GTK_UI_MANAGER_MENUBAR | 
@@ -893,7 +859,7 @@ get_child_node (GtkUIManager *self,
 	{
 	  Node *mnode;
 	  
-	  mnode = g_chunk_new0 (Node, merge_node_chunk);
+	  mnode = g_slice_new0 (Node);
 	  mnode->type = node_type;
 	  mnode->name = g_strndup (childname, childname_length);
 
@@ -933,7 +899,7 @@ get_child_node (GtkUIManager *self,
 	{
 	  Node *mnode;
 
-	  mnode = g_chunk_new0 (Node, merge_node_chunk);
+	  mnode = g_slice_new0 (Node);
 	  mnode->type = node_type;
 	  mnode->name = g_strndup (childname, childname_length);
 	  mnode->dirty = TRUE;
@@ -986,12 +952,18 @@ get_node (GtkUIManager *self,
   return node;
 }
 
+static void
+node_ui_reference_free (gpointer data)
+{
+  g_slice_free (NodeUIReference, data);
+}
+
 static gboolean
 free_node (GNode *node)
 {
   Node *info = NODE_INFO (node);
   
-  g_list_foreach (info->uifiles, (GFunc) g_free, NULL);
+  g_list_foreach (info->uifiles, (GFunc) node_ui_reference_free, NULL);
   g_list_free (info->uifiles);
 
   if (info->action)
@@ -1001,7 +973,7 @@ free_node (GNode *node)
   if (info->extra)
     g_object_unref (info->extra);
   g_free (info->name);
-  g_chunk_free (info, merge_node_chunk);
+  g_slice_free (Node, info);
 
   return FALSE;
 }
@@ -1038,7 +1010,7 @@ node_prepend_ui_reference (GNode  *gnode,
     reference = node->uifiles->data;
   else
     {
-      reference = g_new (NodeUIReference, 1);
+      reference = g_slice_new (NodeUIReference);
       node->uifiles = g_list_prepend (node->uifiles, reference);
     }
 
@@ -1064,7 +1036,7 @@ node_remove_ui_reference (GNode  *gnode,
 	  if (p == node->uifiles)
 	    mark_node_dirty (gnode);
 	  node->uifiles = g_list_delete_link (node->uifiles, p);
-	  g_free (reference);
+	  g_slice_free (NodeUIReference, reference);
 
 	  break;
 	}
@@ -1305,7 +1277,7 @@ start_element_handler (GMarkupParseContext *context,
 	  if (!strcmp (node_name, "separator"))
 	    {
 	      node_name = NULL;
-	      length = -1;
+	      length = 0;
 	    }
 	  else
 	    length = strlen (node_name);
@@ -1464,7 +1436,7 @@ text_handler (GMarkupParseContext *context,
 }
 
 
-static GMarkupParser ui_parser = {
+static const GMarkupParser ui_parser = {
   start_element_handler,
   end_element_handler,
   text_handler,
@@ -1820,15 +1792,15 @@ find_menu_position (GNode      *node,
 		    gint       *pos_p)
 {
   GtkWidget *menushell;
-  gint pos;
+  gint pos = 0;
 
   g_return_val_if_fail (node != NULL, FALSE);
   g_return_val_if_fail (NODE_INFO (node)->type == NODE_TYPE_MENU ||
-			NODE_INFO (node)->type == NODE_TYPE_POPUP ||
-			NODE_INFO (node)->type == NODE_TYPE_MENU_PLACEHOLDER ||
-			NODE_INFO (node)->type == NODE_TYPE_MENUITEM ||
-			NODE_INFO (node)->type == NODE_TYPE_SEPARATOR,
-			FALSE);
+		        NODE_INFO (node)->type == NODE_TYPE_POPUP ||
+		        NODE_INFO (node)->type == NODE_TYPE_MENU_PLACEHOLDER ||
+		        NODE_INFO (node)->type == NODE_TYPE_MENUITEM ||
+		        NODE_INFO (node)->type == NODE_TYPE_SEPARATOR,
+                        FALSE);
 
   /* first sibling -- look at parent */
   if (node->prev == NULL)
@@ -1862,8 +1834,8 @@ find_menu_position (GNode      *node,
 			      NODE_INFO (parent)->proxy) + 1;
 	  break;
 	default:
-	  g_warning("%s: bad parent node type %d", G_STRLOC,
-		    NODE_INFO (parent)->type);
+	  g_warning ("%s: bad parent node type %d", G_STRLOC,
+		     NODE_INFO (parent)->type);
 	  return FALSE;
 	}
     }
@@ -1878,9 +1850,12 @@ find_menu_position (GNode      *node,
       else
 	prev_child = NODE_INFO (sibling)->proxy;
 
-      g_return_val_if_fail (GTK_IS_WIDGET (prev_child), FALSE);
+      if (!GTK_IS_WIDGET (prev_child))
+        return FALSE;
+
       menushell = gtk_widget_get_parent (prev_child);
-      g_return_val_if_fail (GTK_IS_MENU_SHELL (menushell), FALSE);
+      if (!GTK_IS_MENU_SHELL (menushell))
+        return FALSE;
 
       pos = g_list_index (GTK_MENU_SHELL (menushell)->children, prev_child) + 1;
     }
@@ -1903,11 +1878,11 @@ find_toolbar_position (GNode      *node,
 
   g_return_val_if_fail (node != NULL, FALSE);
   g_return_val_if_fail (NODE_INFO (node)->type == NODE_TYPE_TOOLBAR ||
-			NODE_INFO (node)->type == NODE_TYPE_TOOLBAR_PLACEHOLDER ||
-			NODE_INFO (node)->type == NODE_TYPE_TOOLITEM ||
-			NODE_INFO (node)->type == NODE_TYPE_SEPARATOR,
-			FALSE);
-
+		        NODE_INFO (node)->type == NODE_TYPE_TOOLBAR_PLACEHOLDER ||
+		        NODE_INFO (node)->type == NODE_TYPE_TOOLITEM ||
+		        NODE_INFO (node)->type == NODE_TYPE_SEPARATOR,
+                        FALSE);
+  
   /* first sibling -- look at parent */
   if (node->prev == NULL)
     {
@@ -1943,14 +1918,17 @@ find_toolbar_position (GNode      *node,
       else
 	prev_child = NODE_INFO (sibling)->proxy;
 
-      g_return_val_if_fail (GTK_IS_WIDGET (prev_child), FALSE);
+      if (!GTK_IS_WIDGET (prev_child))
+        return FALSE;
+
       toolbar = gtk_widget_get_parent (prev_child);
-      g_return_val_if_fail (GTK_IS_TOOLBAR (toolbar), FALSE);
+      if (!GTK_IS_TOOLBAR (toolbar))
+        return FALSE;
 
       pos = gtk_toolbar_get_item_index (GTK_TOOLBAR (toolbar),
 					GTK_TOOL_ITEM (prev_child)) + 1;
     }
-
+  
   if (toolbar_p)
     *toolbar_p = toolbar;
   if (pos_p)
@@ -1975,6 +1953,7 @@ gboolean
 _gtk_menu_is_empty (GtkWidget *menu)
 {
   GList *children, *cur;
+  gboolean result = TRUE;
 
   g_return_val_if_fail (menu == NULL || GTK_IS_MENU (menu), TRUE);
 
@@ -1990,13 +1969,16 @@ _gtk_menu_is_empty (GtkWidget *menu)
 	{
 	  if (!GTK_IS_TEAROFF_MENU_ITEM (cur->data) &&
 	      !g_object_get_data (cur->data, "gtk-empty-menu-item"))
-	    return FALSE;
+            {
+	      result = FALSE;
+              break;
+            }
 	}
       cur = cur->next;
     }
   g_list_free (children);
 
-  return TRUE;
+  return result;
 }
 
 enum {
@@ -2005,10 +1987,6 @@ enum {
   SEPARATOR_MODE_HIDDEN
 };
 
-void _gtk_action_sync_menu_visible (GtkAction *action,
-				    GtkWidget *proxy,
-				    gboolean   empty);
-
 static void
 update_smart_separators (GtkWidget *proxy)
 {
@@ -2025,7 +2003,6 @@ update_smart_separators (GtkWidget *proxy)
       gboolean empty;
       GList *children, *cur, *last;
       GtkWidget *filler;
-      gint i;
 
       children = gtk_container_get_children (GTK_CONTAINER (parent));
       
@@ -2034,7 +2011,6 @@ update_smart_separators (GtkWidget *proxy)
       empty = TRUE;
       filler = NULL;
 
-      i = 0;
       cur = children;
       while (cur) 
 	{
@@ -2169,9 +2145,9 @@ update_node (GtkUIManager *self,
       info->type != NODE_TYPE_MENU_PLACEHOLDER &&
       info->type != NODE_TYPE_TOOLBAR_PLACEHOLDER)
     {
-      g_warning ("%s: missing action", info->name);
+      g_warning ("%s: missing action %s", info->name, action_name);
       
-      goto recurse_children;
+      return;
     }
   
   if (action)
@@ -2211,8 +2187,7 @@ update_node (GtkUIManager *self,
       if (info->proxy == NULL)
 	{
 	  info->proxy = gtk_menu_bar_new ();
-	  g_object_ref (info->proxy);
-	  gtk_object_sink (GTK_OBJECT (info->proxy));
+	  g_object_ref_sink (info->proxy);
 	  gtk_widget_set_name (info->proxy, info->name);
 	  gtk_widget_show (info->proxy);
 	  g_signal_emit (self, ui_manager_signals[ADD_WIDGET], 0, info->proxy);
@@ -2222,8 +2197,7 @@ update_node (GtkUIManager *self,
       if (info->proxy == NULL) 
 	{
 	  info->proxy = gtk_menu_new ();
-	  g_object_ref (info->proxy);
-	  gtk_object_sink (GTK_OBJECT (info->proxy));
+	  g_object_ref_sink (info->proxy);
 	}
       gtk_widget_set_name (info->proxy, info->name);
       break;
@@ -2255,48 +2229,45 @@ update_node (GtkUIManager *self,
 	/* create proxy if needed ... */
 	if (info->proxy == NULL)
 	  {
-	    GtkWidget *menushell;
-	    gint pos;
+	    GtkWidget *tearoff;
+	    GtkWidget *filler;
 	    
-	    if (NODE_INFO (node->parent)->type == NODE_TYPE_TOOLITEM ||
-		find_menu_position (node, &menushell, &pos))
+	    menu = gtk_menu_new ();
+	    gtk_widget_set_name (menu, info->name);
+	    tearoff = gtk_tearoff_menu_item_new ();
+	    gtk_widget_set_no_show_all (tearoff, TRUE);
+	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), tearoff);
+	    filler = gtk_menu_item_new_with_label (_("Empty"));
+	    g_object_set_data (G_OBJECT (filler),
+			       I_("gtk-empty-menu-item"),
+			       GINT_TO_POINTER (TRUE));
+	    gtk_widget_set_sensitive (filler, FALSE);
+	    gtk_widget_set_no_show_all (filler, TRUE);
+	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), filler);
+	    
+	    if (NODE_INFO (node->parent)->type == NODE_TYPE_TOOLITEM)
 	      {
-		GtkWidget *tearoff;
-		GtkWidget *filler;
+		info->proxy = menu;
+		g_object_ref_sink (info->proxy);
+		gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (NODE_INFO (node->parent)->proxy),
+					       menu);
+	      }
+	    else
+	      {
+		GtkWidget *menushell;
+		gint pos;
 		
-		menu = gtk_menu_new ();
-		gtk_widget_set_name (menu, info->name);
-		tearoff = gtk_tearoff_menu_item_new ();
-		gtk_widget_set_no_show_all (tearoff, TRUE);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), tearoff);
-		filler = gtk_menu_item_new_with_label (_("Empty"));
-		g_object_set_data (G_OBJECT (filler),
-				   "gtk-empty-menu-item",
-				   GINT_TO_POINTER (TRUE));
-		gtk_widget_set_sensitive (filler, FALSE);
-		gtk_widget_set_no_show_all (filler, TRUE);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), filler);
-
-		if (NODE_INFO (node->parent)->type == NODE_TYPE_TOOLITEM)
-		  {
-		    info->proxy = menu;
-		    g_object_ref (info->proxy);
-		    gtk_object_sink (GTK_OBJECT (info->proxy));
-		    gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (NODE_INFO (node->parent)->proxy),
-						   menu);
-		  }
-		else
-		  {
-		    info->proxy = gtk_action_create_menu_item (action);
-		    g_object_ref (info->proxy);
-		    gtk_object_sink (GTK_OBJECT (info->proxy));
-		    g_signal_connect (info->proxy, "notify::visible",
-				      G_CALLBACK (update_smart_separators), NULL);
-		    gtk_widget_set_name (info->proxy, info->name);
-		    
-		    gtk_menu_item_set_submenu (GTK_MENU_ITEM (info->proxy), menu);
-		    gtk_menu_shell_insert (GTK_MENU_SHELL (menushell), info->proxy, pos);
-		  }
+		if (find_menu_position (node, &menushell, &pos))
+                  {
+		     info->proxy = gtk_action_create_menu_item (action);
+		     g_object_ref_sink (info->proxy);
+		     g_signal_connect (info->proxy, "notify::visible",
+		   		       G_CALLBACK (update_smart_separators), NULL);
+		     gtk_widget_set_name (info->proxy, info->name);
+		
+		     gtk_menu_item_set_submenu (GTK_MENU_ITEM (info->proxy), menu);
+		     gtk_menu_shell_insert (GTK_MENU_SHELL (menushell), info->proxy, pos);
+                 }
 	      }
 	  }
 	else
@@ -2333,8 +2304,7 @@ update_node (GtkUIManager *self,
       if (info->proxy == NULL)
 	{
 	  info->proxy = gtk_toolbar_new ();
-	  g_object_ref (info->proxy);
-	  gtk_object_sink (GTK_OBJECT (info->proxy));
+	  g_object_ref_sink (info->proxy);
 	  gtk_widget_set_name (info->proxy, info->name);
 	  gtk_widget_show (info->proxy);
 	  g_signal_emit (self, ui_manager_signals[ADD_WIDGET], 0, info->proxy);
@@ -2366,27 +2336,25 @@ update_node (GtkUIManager *self,
 	  gint pos;
 	  
 	  if (find_menu_position (node, &menushell, &pos))
-	    {
+            {
 	      info->proxy = gtk_separator_menu_item_new ();
-	      g_object_ref (info->proxy);
-	      gtk_object_sink (GTK_OBJECT (info->proxy));
+	      g_object_ref_sink (info->proxy);
 	      g_object_set_data (G_OBJECT (info->proxy),
-				 "gtk-separator-mode",
-				 GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
+	  		         I_("gtk-separator-mode"),
+			         GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
 	      gtk_widget_set_no_show_all (info->proxy, TRUE);
 	      gtk_menu_shell_insert (GTK_MENU_SHELL (menushell),
-				     NODE_INFO (node)->proxy, pos);
-	      
+	 			     NODE_INFO (node)->proxy, pos);
+	  
 	      info->extra = gtk_separator_menu_item_new ();
-	      g_object_ref (info->extra);
-	      gtk_object_sink (GTK_OBJECT (info->extra));
+	      g_object_ref_sink (info->extra);
 	      g_object_set_data (G_OBJECT (info->extra),
-				 "gtk-separator-mode",
-				 GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
+			         I_("gtk-separator-mode"),
+			         GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
 	      gtk_widget_set_no_show_all (info->extra, TRUE);
 	      gtk_menu_shell_insert (GTK_MENU_SHELL (menushell),
-				     NODE_INFO (node)->extra, pos+1);
-	    }
+				     NODE_INFO (node)->extra, pos + 1);
+            }
 	}
       break;
     case NODE_TYPE_TOOLBAR_PLACEHOLDER:
@@ -2413,31 +2381,28 @@ update_node (GtkUIManager *self,
 	{
 	  GtkWidget *toolbar;
 	  gint pos;
+	  GtkToolItem *item;    
 	  
 	  if (find_toolbar_position (node, &toolbar, &pos))
-	    {
-	      GtkToolItem *item;
-	      
+            {
 	      item = gtk_separator_tool_item_new ();
 	      gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, pos);
 	      info->proxy = GTK_WIDGET (item);
-	      g_object_ref (info->proxy);
-	      gtk_object_sink (GTK_OBJECT (info->proxy));
+	      g_object_ref_sink (info->proxy);
 	      g_object_set_data (G_OBJECT (info->proxy),
-				 "gtk-separator-mode",
-				 GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
+			         I_("gtk-separator-mode"),
+			         GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
 	      gtk_widget_set_no_show_all (info->proxy, TRUE);
-	      
+	  
 	      item = gtk_separator_tool_item_new ();
 	      gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, pos+1);
 	      info->extra = GTK_WIDGET (item);
-	      g_object_ref (info->extra);
-	      gtk_object_sink (GTK_OBJECT (info->extra));
+	      g_object_ref_sink (info->extra);
 	      g_object_set_data (G_OBJECT (info->extra),
-				 "gtk-separator-mode",
-				 GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
+			         I_("gtk-separator-mode"),
+			         GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
 	      gtk_widget_set_no_show_all (info->extra, TRUE);
-	    }
+            }
 	}
       break;
     case NODE_TYPE_MENUITEM:
@@ -2461,15 +2426,14 @@ update_node (GtkUIManager *self,
 	  gint pos;
 	  
 	  if (find_menu_position (node, &menushell, &pos))
-	    {
+            {
 	      info->proxy = gtk_action_create_menu_item (action);
-	      g_object_ref (info->proxy);
-	      gtk_object_sink (GTK_OBJECT (info->proxy));
+	      g_object_ref_sink (info->proxy);
 	      gtk_widget_set_name (info->proxy, info->name);
-	      
+	  
 	      gtk_menu_shell_insert (GTK_MENU_SHELL (menushell),
 				     info->proxy, pos);
-	    }
+           }
 	}
       else
 	{
@@ -2479,16 +2443,18 @@ update_node (GtkUIManager *self,
 	  gtk_menu_item_set_submenu (GTK_MENU_ITEM (info->proxy), NULL);
 	  gtk_action_connect_proxy (action, info->proxy);
 	}
-      g_signal_connect (info->proxy, "notify::visible",
-			G_CALLBACK (update_smart_separators), NULL);
-      if (in_popup) 
-	{
-	  /* don't show accels in popups */
-	  GtkWidget *label = GTK_BIN (info->proxy)->child;
-	  g_object_set (label,
-			"accel_closure", NULL,
-			NULL);
-	}
+
+      if (info->proxy)
+        {
+          g_signal_connect (info->proxy, "notify::visible",
+			    G_CALLBACK (update_smart_separators), NULL);
+          if (in_popup) 
+	    {
+	      /* don't show accels in popups */
+	      GtkWidget *label = GTK_BIN (info->proxy)->child;
+	      g_object_set (label, "accel-closure", NULL, NULL);
+	    }
+        }
       
       break;
     case NODE_TYPE_TOOLITEM:
@@ -2512,15 +2478,14 @@ update_node (GtkUIManager *self,
 	  gint pos;
 	  
 	  if (find_toolbar_position (node, &toolbar, &pos))
-	    {
+            {
 	      info->proxy = gtk_action_create_tool_item (action);
-	      g_object_ref (info->proxy);
-	      gtk_object_sink (GTK_OBJECT (info->proxy));
+	      g_object_ref_sink (info->proxy);
 	      gtk_widget_set_name (info->proxy, info->name);
 	      
 	      gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
-				  GTK_TOOL_ITEM (info->proxy), pos);
-	    }
+	  		          GTK_TOOL_ITEM (info->proxy), pos);
+            }
 	}
       else
 	{
@@ -2530,14 +2495,21 @@ update_node (GtkUIManager *self,
 	  gtk_action_connect_proxy (action, info->proxy);
 	}
 
-      /* FIXME: we must trigger the notify::tooltip handler, since 
-       * tooltips on toolitems can't be set before the toolitem 
-       * is added to the toolbar.
-       */
-      g_object_notify (G_OBJECT (action), "tooltip");
+      if (info->proxy)
+        {
+          /* FIXME: we must re-set the tooltip, since tooltips on 
+           * toolitems can't be set before the toolitem is added 
+           * to the toolbar.
+           */
+	  gchar *tooltip;
 
-      g_signal_connect (info->proxy, "notify::visible",
-			G_CALLBACK (update_smart_separators), NULL);
+	  g_object_get (G_OBJECT (action), "tooltip", &tooltip, NULL);
+	  g_object_set (G_OBJECT (action), "tooltip", tooltip, NULL);
+	  g_free (tooltip);
+     
+          g_signal_connect (info->proxy, "notify::visible",
+			    G_CALLBACK (update_smart_separators), NULL);
+        }
       break;
     case NODE_TYPE_SEPARATOR:
       if (NODE_INFO (node->parent)->type == NODE_TYPE_TOOLBAR ||
@@ -2546,7 +2518,8 @@ update_node (GtkUIManager *self,
 	  GtkWidget *toolbar;
 	  gint pos;
 	  gint separator_mode;
-	  
+	  GtkToolItem *item;
+
 	  if (GTK_IS_SEPARATOR_TOOL_ITEM (info->proxy))
 	    {
 	      gtk_container_remove (GTK_CONTAINER (info->proxy->parent),
@@ -2556,28 +2529,26 @@ update_node (GtkUIManager *self,
 	    }
 	  
 	  if (find_toolbar_position (node, &toolbar, &pos))
-	    {
-	      GtkToolItem *item = gtk_separator_tool_item_new ();
+            {
+	      item  = gtk_separator_tool_item_new ();
 	      gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, pos);
 	      info->proxy = GTK_WIDGET (item);
-	      g_object_ref (info->proxy);
-	      gtk_object_sink (GTK_OBJECT (info->proxy));
+	      g_object_ref_sink (info->proxy);
 	      gtk_widget_set_no_show_all (info->proxy, TRUE);
 	      if (info->expand)
-		{
-		  gtk_tool_item_set_expand (GTK_TOOL_ITEM (item), TRUE);
-		  gtk_separator_tool_item_set_draw
-		    (GTK_SEPARATOR_TOOL_ITEM (item), FALSE);
-		  separator_mode = SEPARATOR_MODE_VISIBLE;
-		}
-		else
-		  separator_mode = SEPARATOR_MODE_SMART;
-
+	        {
+	          gtk_tool_item_set_expand (GTK_TOOL_ITEM (item), TRUE);
+	          gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM (item), FALSE);
+	          separator_mode = SEPARATOR_MODE_VISIBLE;
+	        }
+	      else
+	        separator_mode = SEPARATOR_MODE_SMART;
+	  
 	      g_object_set_data (G_OBJECT (info->proxy),
-				 "gtk-separator-mode",
-				 GINT_TO_POINTER (separator_mode));
+			         I_("gtk-separator-mode"),
+			         GINT_TO_POINTER (separator_mode));
 	      gtk_widget_show (info->proxy);
-	    }
+            }
 	}
       else
 	{
@@ -2594,17 +2565,16 @@ update_node (GtkUIManager *self,
 	  
 	  if (find_menu_position (node, &menushell, &pos))
 	    {
-	      info->proxy = gtk_separator_menu_item_new ();
-	      g_object_ref (info->proxy);
-	      gtk_object_sink (GTK_OBJECT (info->proxy));
+              info->proxy = gtk_separator_menu_item_new ();
+	      g_object_ref_sink (info->proxy);
 	      gtk_widget_set_no_show_all (info->proxy, TRUE);
 	      g_object_set_data (G_OBJECT (info->proxy),
-				 "gtk-separator-mode",
-				 GINT_TO_POINTER (SEPARATOR_MODE_SMART));
+			         I_("gtk-separator-mode"),
+			         GINT_TO_POINTER (SEPARATOR_MODE_SMART));
 	      gtk_menu_shell_insert (GTK_MENU_SHELL (menushell),
 				     info->proxy, pos);
 	      gtk_widget_show (info->proxy);
-	    }
+            }
 	}
       break;
     case NODE_TYPE_ACCELERATOR:
@@ -2619,7 +2589,6 @@ update_node (GtkUIManager *self,
   info->action = action;
 
  recurse_children:
-  /* process children */
   child = node->children;
   while (child)
     {
@@ -2758,35 +2727,43 @@ mark_node_dirty (GNode *node)
     NODE_INFO (p)->dirty = TRUE;  
 }
 
-static const gchar *const open_tag_format[] = {
-  "%*s<UNDECIDED",
-  "%*s<ui",
-  "%*s<menubar",
-  "%*s<menu",
-  "%*s<toolbar",
-  "%*s<placeholder",
-  "%*s<placeholder",
-  "%*s<popup",
-  "%*s<menuitem",
-  "%*s<toolitem",
-  "%*s<separator",
-  "%*s<accelerator"
-};
+static const gchar *
+open_tag_format (NodeType type)
+{
+  switch (type)
+    {
+    case NODE_TYPE_UNDECIDED: return "%*s<UNDECIDED"; 
+    case NODE_TYPE_ROOT: return "%*s<ui"; 
+    case NODE_TYPE_MENUBAR: return "%*s<menubar";
+    case NODE_TYPE_MENU: return "%*s<menu";
+    case NODE_TYPE_TOOLBAR: return "%*s<toolbar";
+    case NODE_TYPE_MENU_PLACEHOLDER:
+    case NODE_TYPE_TOOLBAR_PLACEHOLDER: return "%*s<placeholder";
+    case NODE_TYPE_POPUP: return "%*s<popup";
+    case NODE_TYPE_MENUITEM: return "%*s<menuitem";
+    case NODE_TYPE_TOOLITEM: return "%*s<toolitem";
+    case NODE_TYPE_SEPARATOR: return "%*s<separator";
+    case NODE_TYPE_ACCELERATOR: return "%*s<accelerator";
+    default: return NULL;
+    }
+}
 
-static const gchar *const close_tag_format[] = {
-  "%*s</UNDECIDED>\n",
-  "%*s</ui>\n",
-  "%*s</menubar>\n",
-  "%*s</menu>\n",
-  "%*s</toolbar>\n",
-  "%*s</placeholder>\n",
-  "%*s</placeholder>\n",
-  "%*s</popup>\n",
-  NULL,
-  NULL,
-  NULL,
-  NULL
-};
+static const gchar *
+close_tag_format (NodeType type)
+{
+  switch (type)
+    {
+    case NODE_TYPE_UNDECIDED: return "%*s</UNDECIDED>\n";
+    case NODE_TYPE_ROOT: return "%*s</ui>\n";
+    case NODE_TYPE_MENUBAR: return "%*s</menubar>\n";
+    case NODE_TYPE_MENU: return "%*s</menu>\n";
+    case NODE_TYPE_TOOLBAR: return "%*s</toolbar>\n";
+    case NODE_TYPE_MENU_PLACEHOLDER:
+    case NODE_TYPE_TOOLBAR_PLACEHOLDER: return "%*s</placeholder>\n";
+    case NODE_TYPE_POPUP: return "%*s</popup>\n";
+    default: return NULL;
+    }
+}
 
 static void
 print_node (GtkUIManager *self,
@@ -2796,11 +2773,15 @@ print_node (GtkUIManager *self,
 {
   Node  *mnode;
   GNode *child;
+  const gchar *open_fmt;
+  const gchar *close_fmt;
 
   mnode = node->data;
 
-  g_string_append_printf (buffer, open_tag_format[mnode->type],
-			  indent_level, "");
+  open_fmt = open_tag_format (mnode->type);
+  close_fmt = close_tag_format (mnode->type);
+
+  g_string_append_printf (buffer, open_fmt, indent_level, "");
 
   if (mnode->type != NODE_TYPE_ROOT)
     {
@@ -2812,15 +2793,13 @@ print_node (GtkUIManager *self,
 				g_quark_to_string (mnode->action_name));
     }
 
-  g_string_append (buffer,
-                   close_tag_format[mnode->type] ? ">\n" : "/>\n");
+  g_string_append (buffer, close_fmt ? ">\n" : "/>\n");
 
   for (child = node->children; child != NULL; child = child->next)
     print_node (self, child, indent_level + 2, buffer);
 
-  if (close_tag_format[mnode->type])
-    g_string_append_printf (buffer, close_tag_format[mnode->type],
-                            indent_level, "");
+  if (close_fmt)
+    g_string_append_printf (buffer, close_fmt, indent_level, "");
 }
 
 /**
